<?php
// $Id: regtag_helper.inc,v 1.14.2.3 2007/01/17 16:36:42 profix898 Exp $

/**
 * function _reptag_table_load().
 * (load tags from table)
 */
function _reptag_table_load($iid, $cleanlevel = 0) {
  // build query
  $query = 'SELECT * FROM {reptag_vars} WHERE uid = \''.intval($iid[0]).'\'';
  for ($i = 1; $i < count($iid); $i++) {
    $query .= ' OR uid = \''.intval($iid[$i]).'\'';
  }
  if (variable_get('reptag_tableprio', 0)) {
    $query .= ' ORDER BY uid';
  }
  else {
    $query .= ' ORDER BY uid DESC';
  }
  // load and format tags
  $vars = array();
  $result = db_query($query);
  while ($var = db_fetch_object($result)) {
    switch ($cleanlevel) {
    case 1:
      $vars['$'.substr($var->name, 3, -5).'$'] = unserialize($var->value);
      break;
    case 2:
      $vars[substr($var->name, 3, -5)] = unserialize($var->value);
      break;
    case 0:
    default:
      $vars[$var->name] = unserialize($var->value);
    }
  }
   
  return $vars;
}

/**
 * function _reptag_table_set().
 * (store tag to table)
 */
function _reptag_table_set($name, $value, $uid) {
  db_lock_table('reptag_vars');
  db_query("DELETE FROM {reptag_vars} WHERE name = '%s' AND uid = '%d'", $name, $uid);
  db_query("INSERT INTO {reptag_vars} (name, value, uid) VALUES ('%s', '%s', '%d')", $name, serialize($value), $uid);
  db_unlock_tables();
}

/**
 * function _reptag_table_del().
 * (remove tag from table)
 */
function _reptag_table_del($name, $uid) {
  db_lock_table('reptag_vars');
  db_query("DELETE FROM {reptag_vars} WHERE name = '%s' AND uid = '%d'", $name, $uid);
  db_unlock_tables();
}

/**
 * function _reptag_table_clear().
 * (remove all tags for a specified user)
 */
function _reptag_table_clear($uid) {
  db_lock_table('reptag_vars');
  db_query("DELETE FROM {reptag_vars} WHERE uid = '%d'", $uid);
  db_unlock_tables();
}

/**
 * function _reptag_roles_load().
 * (load all available roles)
 */
function _reptag_roles_load() {
  $roles = array();
  $result = db_query("SELECT * FROM {role}");
  while ($var = db_fetch_object($result)) {
    $roles[$var->name] = $var->rid;
  }
  
  return $roles;
}

/**
 * function _reptag_tags_implode().
 */
function _reptag_tags_implode($tags) {
  $tags = array_filter($tags);
  $tags = implode('|', $tags);
  return $tags;
}

/**
 * function _reptag_exclude_loadtags().
 * (load exclude tags from variable)
 */
function _reptag_exclude_loadtags($rid) {
  $tags = unserialize(variable_get('reptag_exclude_r'.$rid, serialize(array())));
  return $tags;
}

/**
 * function _reptag_exclude_loadtags_string().
 * (load exclude tags from variable into a string)
 */
function _reptag_exclude_loadtags_string($rid) {
  $tags = _reptag_exclude_loadtags($rid);
  $tags = _reptag_tags_implode($tags);
  return $tags;
}

/**
 * function _reptag_exclude_storetags().
 * (store exclude tags to variable)
 */
function _reptag_exclude_storetags($rid, $tags) {
  variable_set('reptag_exclude_r'.$rid, serialize($tags));
}

/**
 * function _reptag_exclude_storetags_string().
 * (explode string and store exclude tags to variable)
 */
function _reptag_exclude_storetags_string($rid, $tags) {
  $tags = explode('|', $tags);
  _reptag_exclude_storetags($rid, $tags);
}

/**
 * function _reptag_user_get_roles().
 * (get roles for a specified user)
 */
function _reptag_user_get_roles($uid) {
  $roles = array();
  switch ($uid) {
  case 0: //anonymous user
    $roles[] = DRUPAL_ANONYMOUS_RID;
  	break;
  case 1: //administrator
    $roles[] = DRUPAL_AUTHENTICATED_RID;  
    $result = db_query("SELECT DISTINCT rid FROM {users_roles}");
    while ($rid = db_fetch_object($result)) {
      $roles[] = $rid->rid;
    }	
  	break;
  default: //authenticated user
    $roles[] = DRUPAL_AUTHENTICATED_RID;
    $result = db_query("SELECT rid FROM {users_roles} WHERE uid = '%d'", $uid);
    while ($rid = db_fetch_object($result)) {
      $roles[] = $rid->rid;
    }
  	break;
  }
  
  return $roles;
}

/**
 * function _reptag_cache_set().
 * (store node into reptag cache)
 */
function _reptag_cache_set($node, $fields) {
  $cache = array();
  foreach ($fields as $field) {
    $cache[$field] = _reptag_content_field($node, $field);
  }
  if (count($cache)) {
    $cid = 'n'.$node->nid.':'.($node->vid ? 'v'.$node->vid : 'c'.$node->cid).':'.md5(serialize($fields));
    cache_set($cid, 'cache_reptag', serialize($cache), variable_get('reptag_cache_lifetime', CACHE_PERMANENT));
  }
}

/**
 * function _reptag_cache_get().
 * (retrieve a node from reptag cache)
 */
function _reptag_cache_get($node, $fields) {
  $cid = 'n'.$node->nid.':'.($node->vid ? 'v'.$node->vid : 'c'.$node->cid).':'.md5(serialize($fields));
  if ($cache = cache_get($cid, 'cache_reptag')) {
    // does cache contain valid data
    if (empty($cache->data)) {
      return FALSE;
    }
    // unserialize data and override fields
    $cache = unserialize($cache->data);
    foreach ($fields as $field) {
      if (isset($cache[$field])) {
        $field_content = &_reptag_content_field($node, $field);
        $field_content = $cache[$field];
      }
    }
    return TRUE;
  }
  
  return FALSE;
}

/**
 * function _reptag_cache_del().
 * (remove a node from reptag cache)
 */
function _reptag_cache_del($node, $fields) {
  $node = (object)$node;
  $cid = 'n'.$node->nid.':'.($node->vid ? 'v'.$node->vid : 'c'.$node->cid).':'.md5(serialize($fields));
  cache_set($cid, 'cache_reptag', '', CACHE_TEMPORARY);
}

/**
 * function _reptag_match_tags().
 * (filter $tags containing $string)
 */
function _reptag_match_tags($string, $tags) {
  $matches = array();
  foreach ($tags as $tag) {
    if (!(strpos($tag, $string) === false)) {
      $matches[$tag] = check_plain($tag);
    }
  }
  
  return $matches;
}

/**
 * function _reptag_content_field().
 * (point $text to the $field path of $content)
 */
function &_reptag_content_field($content, $field) {
  $text = &$content;
  $elements = explode('/', $field);
  foreach ($elements as $element) {
    if (is_array($text)) {
      $text = &$text[$element];
    }
    elseif (is_object($text)) {
      $text = &$text->$element;
    }
    else {
      return NULL;
    }
  }
  
  return $text;
}

/**
 * function _reptag_content_field_path().
 * (find path to a certain field)
 */
function _reptag_content_field_path(&$field, $field_info, $type) {
  $field = 'content/'.$field.'/0/#value';
  // todo: generic way to find the path to a field
}

?>
