<?php
// $Id: reptag_ajax.inc,v 1.9 2007/01/03 23:31:57 profix898 Exp $

/**
 * function _reptag_ajax_callback().
 */
function _reptag_ajax_callback($query) {
  // recover query string
  list($op, $args) = explode('?', $query);
  parse_str(urldecode($args), $args);
  _reptag_sanitize_args($args);
  // call ajax support routines (add, remove, edit, ...)
  switch ($op) {
    case 'tagadd':
      $result = _reptag_ajax_tagadd($args);
      break;
    case 'tagremove':
      $result = _reptag_ajax_tagremove($args);
      break;
    case 'tagedit':
      $result = _reptag_ajax_tagedit($args);
      break;
    case 'excludeadd':
      $result = _reptag_ajax_excludeadd($args);
      break;
    case 'excluderemove':
      $result = _reptag_ajax_excluderemove($args);
      break;
    case 'excludelist':
      $result = _reptag_ajax_excludelist($args);
      break;
    case 'tagadd_window':
      print _reptag_ajax_tagadd_window($args);
      exit();
    default:
      $result = array('status' => 'noop');
  }
  $result['op'] = $op;
  $result['id'] = $args['id'];
  print drupal_to_js($result);
  exit();
}

/**
 * function _reptag_sanitize_args().
 * (filter args to remove malicious content)
 */
function _reptag_sanitize_args(&$args) {
  // filter allowed tags
  $allowed = array('id' => 1, 'name' => 1, 'value' => 1, 'uid' => 1);
  foreach ($args as $key => $value) {
    if (!isset($allowed[$key])) {
      unset($args[$key]);
    }
  }
  // validate values
  $args['id'] = intval($args['id']);
  $args['name'] = strtoupper($args['name']);
  $args['value'] = $args['value'];
  $args['uid'] = intval($args['uid']);
}

/**
 * function _reptag_ajax_tagadd().
 */
function _reptag_ajax_tagadd($args) {
  if (!empty($args['name']) && !empty($args['value'])) {
    // load existing tags
    $reptag_table = array_keys(_reptag_table_load(array($args['uid']), 2));
    if (!in_array($args['name'], $reptag_table)) {
      // tag does not exist => add tag
      _reptag_table_set('#\$'.$args['name'].'\$#si', $args['value'], $args['uid']);
      return array('status' => 'success');
    }
    else {
      // tag already exists
      return array('status' => 'failed', 'message' => t('Error: Tag already exists'));
    }
  }
  
  return array('status' => 'failed', 'message' => t('Error: Empty field(s)'));
}

/**
 * function _reptag_ajax_tagremove().
 */
function _reptag_ajax_tagremove($args) {
  if (!empty($args['name'])) {
    // load existing tags
    $reptag_table = array_keys(_reptag_table_load(array($args['uid']), 2));
    if (in_array($args['name'], $reptag_table)) {
      // tag exists => remove tag
      _reptag_table_del('#\$'.$args['name'].'\$#si', $args['uid']);
      return array('status' => 'success');
    }
    else {
      // tag does not exist
      return array('status' => 'failed', 'message' => t('Error: Tag does not exist'));
    }
    return array('status' => 'success');
  }
  
  return array('status' => 'failed', 'message' => t('Error: Empty field(s)'));
}

/**
 * function _reptag_ajax_tagedit().
 */
function _reptag_ajax_tagedit($args) {
  if (!empty($args['name']) && !empty($args['value'])) {
    // load existing tags
    $reptag_table = _reptag_table_load(array($args['uid']), 2);
    if (in_array($args['name'], array_keys($reptag_table))) {
      // tag exists => replace tag
      _reptag_table_set('#\$'.$args['name'].'\$#si', $args['value'], $args['uid']);
      return array('status' => 'success');
    }
    elseif (in_array($args['value'], $reptag_table)) {
      // replacement exists => find key => replace tag
      $key = array_search($args['value'], $reptag_table);
      _reptag_table_set('#\$'.$key.'\$#si', $args['value'], $args['uid']);
      return array('status' => 'success');
    }
    else {
      // tag does not exist
      return array('status' => 'failed', 'message' => t('Error: Tag does not exist'));
    }
  }
  
  return array('status' => 'failed', 'message' => t('Error: Empty field(s)'));
}

/**
 * function _reptag_ajax_excludeadd().
 */
function _reptag_ajax_excludeadd($args) {
  if (!empty($args['value'])) {
    $exclude_tags = _reptag_exclude_loadtags($args['id']);
    if (!in_array($args['value'], $exclude_tags)) {
      array_push($exclude_tags, strtoupper($args['value']));
      _reptag_exclude_storetags($args['id'], $exclude_tags);
      return array('status' => 'success', 'list' => _reptag_tags_implode($exclude_tags));
    }
    else {
      return array('status' => 'failed', 'message' => t('Error: Tag already in list'));
    }
  }
  
  return array('status' => 'failed', 'message' => t('Error: Empty field(s)'));
}

/**
 * function _reptag_ajax_excluderemove().
 */
function _reptag_ajax_excluderemove($args) {
  if (!empty($args['name'])) {
    $exclude_tags = array_filter(_reptag_exclude_loadtags($args['id']));
    $key = array_search($args['name'], $exclude_tags);
    unset($exclude_tags[$key]);
    _reptag_exclude_storetags($args['id'], $exclude_tags);
    $result = array('status' => 'success', 'data' => '');
    if (count($exclude_tags) > 0) {
      $result = _reptag_ajax_excludelist($args);
    }
    $result['list'] = _reptag_tags_implode($exclude_tags);
    return $result;
  }
  
  return array('status' => 'failed', 'message' => t('Error: Invalid selection'));
}

/**
 * function _reptag_ajax_excludelist().
 * (create a table of existing tags) 
 */
function _reptag_ajax_excludelist($args) {
  $exclude_tags = array_filter(_reptag_exclude_loadtags($args['id']));
  if (count($exclude_tags) > 0) {
    // format header-like row
    $rows[] = array(
      array('data' => t('<em>Click to remove.</em>'), 'colspan' => 2),
      array('data' => '<a href="javascript:reptag_exclude_close();">'.t('Close').'</a>')
    );
    $rows[] = array(array('data' => '', 'colspan' => 4));
    // create tags list/table
    $exclude_tags = array_chunk($exclude_tags, 3, TRUE);
    foreach ($exclude_tags as $tags) {
      $row = array();
      $tag = each($tags);
      $table['colA'] = array(
        '#value' => $tag[1],
        '#prefix' => '<a href="javascript:reptag_exclude_del(\''.$tag[1].'\');">',
        '#suffix' => '</a>'
      );
      $tag = each($tags);
      $table['colB'] = array(
        '#value' => $tag[1],
        '#prefix' => '<a href="javascript:reptag_exclude_del(\''.$tag[1].'\');">',
        '#suffix' => '</a>'
      );
      $tag = each($tags);
      $table['colC'] = array(
        '#value' => $tag[1],
        '#prefix' => '<a href="javascript:reptag_exclude_del(\''.$tag[1].'\');">',
        '#suffix' => '</a>'
      );
      $row[] = drupal_render($table['colA']);
      $row[] = drupal_render($table['colB']);
      $row[] = drupal_render($table['colC']);
      $rows[] = $row;
    }
    $header = array('', '', '');
    return array('status' => 'success', 'data' => '<div id="reptag-exclude-remove">'.theme('table', $header, $rows).'</div>');
  }
  
  return array('status' => 'failed', 'message' => t('Error: Nothing to list'));
}

/**
 * function _reptag_ajax_tagadd_window().
 * (create html form with autocomplete textfield)  
 */
function _reptag_ajax_tagadd_window() {
  drupal_set_title('Rep[lacement]Tags');
  drupal_set_header('Content-Type: text/html; charset=utf-8');
  drupal_add_js(drupal_get_path('module', 'reptag').'/reptag_helper.js');
  $content = drupal_get_form('_reptag_ajax_tagadd_window_form');
  
  $output  = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";
  $output .= '<html xmlns="http://www.w3.org/1999/xhtml">';
  $output .= '<head>';
  $output .= '<title>'. strip_tags(drupal_get_title()) .'</title>';
  $output .= drupal_get_html_head();
  $output .= drupal_get_css();
  $output .= drupal_get_js();
  $output .= '</head>';
  
  $output .= '<body><div class="reptag_tagadd" style="margin: 15px;">';
  $output .= $content;
  $output .= '</div></body>';
  
  $output .= '</html>';
  
  return $output;
}

/**
 * function _reptag_ajax_tagadd_window_form(). 
 */
function _reptag_ajax_tagadd_window_form() {
  $form['textfield'] = array(
    '#type' => 'textfield',
    '#title' => t('Rep[lacement]Tag'),
    '#default_value' => '',
    '#size' => 35,
    '#maxlength' => 128,
    '#autocomplete_path' => 'reptag/autocomplete'
  );
  $form['insertbtn'] = array(
    '#type' => 'reptag_insert_button',
    '#value' => t('Insert'),
    '#name' => 'insert'
  );
  
  return $form;
}

/**
 * function theme_reptag_insert_button().
 */
function theme_reptag_insert_button($element) {
  return '<input type="button" name="'. $element['#name'] .'" value="'. check_plain($element['#value']) .'" '. drupal_attributes($element['#attributes']) ." />\n";
}

?>
